@@ -163,6 +163,7 @@ def test_strict_slashes_redirect():
             r.Rule("/bar/", endpoint="get", methods=["GET"]),
             r.Rule("/bar", endpoint="post", methods=["POST"]),
             r.Rule("/foo/", endpoint="foo", methods=["POST"]),
+            r.Rule("/<path:var>/", endpoint="path", methods=["GET"]),
         ]
     )
     adapter = map.bind("example.org", "/")
@@ -170,6 +171,7 @@ def test_strict_slashes_redirect():
     # Check if the actual routes works
     assert adapter.match("/bar/", method="GET") == ("get", {})
     assert adapter.match("/bar", method="POST") == ("post", {})
+    assert adapter.match("/abc/", method="GET") == ("path", {"var": "abc"})
 
     # Check if exceptions are correct
     pytest.raises(r.RequestRedirect, adapter.match, "/bar", method="GET")
@@ -177,6 +179,9 @@ def test_strict_slashes_redirect():
     with pytest.raises(r.RequestRedirect) as error_info:
         adapter.match("/foo", method="POST")
     assert error_info.value.code == 308
+    with pytest.raises(r.RequestRedirect) as error_info:
+        adapter.match("/abc", method="GET")
+    assert error_info.value.new_url == "http://example.org/abc/"
 
     # Check differently defined order
     map = r.Map(
@@ -773,6 +778,35 @@ def test_converter_with_tuples():
     assert kwargs["foo"] == ("qwert", "yuiop")
 
 
+def test_nested_regex_groups():
+    """
+    Regression test for https://github.com/pallets/werkzeug/issues/2590
+    """
+
+    class RegexConverter(r.BaseConverter):
+        def __init__(self, url_map, *items):
+            super().__init__(url_map)
+            self.part_isolating = False
+            self.regex = items[0]
+
+    # This is a regex pattern with nested groups
+    DATE_PATTERN = r"((\d{8}T\d{6}([.,]\d{1,3})?)|(\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}([.,]\d{1,3})?))Z"  # noqa: B950
+
+    map = r.Map(
+        [
+            r.Rule(
+                f"/<regex('{DATE_PATTERN}'):start>/<regex('{DATE_PATTERN}'):end>/",
+                endpoint="handler",
+            )
+        ],
+        converters={"regex": RegexConverter},
+    )
+    a = map.bind("example.org", "/")
+    route, kwargs = a.match("/2023-02-16T23:36:36.266Z/2023-02-16T23:46:36.266Z/")
+    assert kwargs["start"] == "2023-02-16T23:36:36.266Z"
+    assert kwargs["end"] == "2023-02-16T23:46:36.266Z"
+
+
 def test_anyconverter():
     m = r.Map(
         [
@@ -800,6 +834,20 @@ def test_any_converter_build_validates_value() -> None:
     assert str(exc.value) == "'invalid' is not one of 'patient', 'provider'"
 
 
+def test_part_isolating_default() -> None:
+    class TwoConverter(r.BaseConverter):
+        regex = r"\w+/\w+"
+
+        def to_python(self, value: str) -> t.Any:
+            return value.split("/")
+
+    m = r.Map(
+        [r.Rule("/<two:values>/", endpoint="two")], converters={"two": TwoConverter}
+    )
+    a = m.bind("localhost")
+    assert a.match("/a/b/") == ("two", {"values": ["a", "b"]})
+
+
 @pytest.mark.parametrize(
     ("endpoint", "value", "expect"),
     [
@@ -1071,18 +1119,6 @@ def test_double_defaults(prefix):
     assert a.build("x", {"bar": True}) == f"{prefix}/bar/"
 
 
-def test_building_bytes():
-    m = r.Map(
-        [
-            r.Rule("/<a>", endpoint="a"),
-            r.Rule("/<b>", defaults={"b": b"\x01\x02\x03"}, endpoint="b"),
-        ]
-    )
-    a = m.bind("example.org", "/")
-    assert a.build("a", {"a": b"\x01\x02\x03"}) == "/%01%02%03"
-    assert a.build("b") == "/%01%02%03"
-
-
 def test_host_matching():
     m = r.Map(
         [
@@ -1427,3 +1463,75 @@ def test_weighting():
         "uuid",
         {"value": uuid.UUID("2b5b0911-fdcf-4dd2-921b-28ace88db8a0")},
     )
+
+
+def test_strict_slashes_false():
+    map = r.Map(
+        [
+            r.Rule("/path1", endpoint="leaf_path", strict_slashes=False),
+            r.Rule("/path2/", endpoint="branch_path", strict_slashes=False),
+            r.Rule(
+                "/<path:path>", endpoint="leaf_path_converter", strict_slashes=False
+            ),
+        ],
+    )
+
+    adapter = map.bind("example.org", "/")
+
+    assert adapter.match("/path1", method="GET") == ("leaf_path", {})
+    assert adapter.match("/path1/", method="GET") == ("leaf_path", {})
+    assert adapter.match("/path2", method="GET") == ("branch_path", {})
+    assert adapter.match("/path2/", method="GET") == ("branch_path", {})
+    assert adapter.match("/any", method="GET") == (
+        "leaf_path_converter",
+        {"path": "any"},
+    )
+    assert adapter.match("/any/", method="GET") == (
+        "leaf_path_converter",
+        {"path": "any/"},
+    )
+
+
+def test_invalid_rule():
+    with pytest.raises(ValueError):
+        map_ = r.Map([r.Rule("/<int()>", endpoint="test")])
+        map_.bind("localhost")
+
+
+def test_multiple_converters_per_part():
+    map_ = r.Map(
+        [
+            r.Rule("/v<int:major>.<int:minor>", endpoint="version"),
+        ],
+    )
+    adapter = map_.bind("localhost")
+    assert adapter.match("/v1.2") == ("version", {"major": 1, "minor": 2})
+
+
+def test_static_regex_escape():
+    map_ = r.Map(
+        [
+            r.Rule("/.<int:value>", endpoint="dotted"),
+        ],
+    )
+    adapter = map_.bind("localhost")
+    assert adapter.match("/.2") == ("dotted", {"value": 2})
+    with pytest.raises(NotFound):
+        adapter.match("/a2")
+
+
+class RegexConverter(r.BaseConverter):
+    def __init__(self, url_map, *items):
+        super().__init__(url_map)
+        self.regex = items[0]
+
+
+def test_regex():
+    map_ = r.Map(
+        [
+            r.Rule(r"/<regex('[^/:]+\.[^/:]+'):value>", endpoint="regex"),
+        ],
+        converters={"regex": RegexConverter},
+    )
+    adapter = map_.bind("localhost")
+    assert adapter.match("/asdfsa.asdfs") == ("regex", {"value": "asdfsa.asdfs"})
